@@ -1,32 +1,13 @@
 /*-
- * Copyright (c) 2008, 2009, 2012, 2013, 2016 - 2019, 2021  Peter Pentchev
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * SPDX-FileCopyrightText: Peter Pentchev <roam@ringlet.net>
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <ctype.h>
 #include <err.h>
 #include <fnmatch.h>
+#include <langinfo.h>
+#include <locale.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -75,6 +56,8 @@ static const char	*matchvalue;
 static const char	*output_prefix;
 /* The postfix to use when displaying the variable name */
 static const char	*output_postfix;
+/* Are we currently operating in an UTF-8 LC_CTYPE setting? */
+static bool		 is_utf8;
 
 size_t			 varsalloc, varscount;
 struct var		*vars;
@@ -407,7 +390,7 @@ init(const int argc, char * const * const argv)
 #if defined(HAVE_PCRE) || defined(HAVE_PCRE2)
 	if (xflag) {
 		if (matchvalue)
-			do_pcre_compile(matchvalue, &x_matchvalue, &x_matchvalue_extra, "value");
+			do_pcre_compile(matchvalue, &x_matchvalue, &x_matchvalue_extra, "value", is_utf8);
 		else
 			x_matchvalue = NULL;
 
@@ -420,7 +403,7 @@ init(const int argc, char * const * const argv)
 			err(1, "Could not allocate memory");
 #endif
 		for (size_t t = 0; t < (size_t)margc; t++)
-			do_pcre_compile(margv[t], &x_margv[t], &x_margv_extra[t], "pattern");
+			do_pcre_compile(margv[t], &x_margv[t], &x_margv_extra[t], "pattern", is_utf8);
 	}
 #endif
 
@@ -695,6 +678,11 @@ foundsection(const char * const _sec)
 int
 main(const int argc, char * const * const argv)
 {
+	setlocale(LC_ALL, "");
+	const char * const encoding = nl_langinfo(CODESET);
+	is_utf8 = strcmp(encoding, "UTF-8") == 0 ||
+	    strcmp(encoding, "UTF8") == 0 || strcmp(encoding, "utf-8") == 0 ||
+	    strcmp(encoding, "utf8") == 0;
 
 	init(argc, argv);
 
