@@ -1,44 +1,18 @@
+/*
+ * SPDX-FileCopyrightText: Peter Pentchev <roam@ringlet.net>
+ * SPDX-License-Identifier: BSD-2-Clause
+ */
 //! confget - a command-line configuration file parser.
 //!
 //! Extract the specified variables from a configuration file and
 //! format their values as requested.
-/*
- * Copyright (c) 2021, 2022  Peter Pentchev <roam@ringlet.net>
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-// Activate most of the clippy::restriction lints that we have come across...
-#![warn(clippy::missing_docs_in_private_items)]
-// ...except for these ones.
-#![allow(clippy::implicit_return)]
 
-use std::collections::HashMap;
 use std::process::{ExitCode, Termination};
 
+use anyhow::{bail, ensure, Context, Result};
 use clap::Parser;
-use expect_exit::{Expected, ExpectedWithError};
 
-use confget::defs::{BackendKind, Config, FEATURES};
+use confget::defs::{BackendKind, Config, FileData};
 use confget::format;
 
 /// The exit code from the program itself.
@@ -75,6 +49,7 @@ enum Mode {
 
 /// The command-line option definitions.
 #[derive(Debug, Parser)]
+#[clap(disable_help_flag(true))]
 #[allow(clippy::struct_excessive_bools)]
 struct Cli {
     /// check if the variable is defined in the file
@@ -164,131 +139,142 @@ confget [-f filename] -q sections [-t type]
 confget -q features
 confget -q feature NAME";
 
+/// Figure out whether a backend type supports `-q sections`.
+const fn supports_sections_query(backend: &BackendKind) -> bool {
+    match *backend {
+        #[cfg(feature = "ini-nom")]
+        BackendKind::IniNom => true,
+
+        #[cfg(feature = "ini-regex")]
+        BackendKind::IniRE => true,
+
+        _ => false,
+    }
+}
+
 /// Process the command-line options, check for conflicts or inconsistencies.
 /// If the `--version`, `--help`, or `-q features/feature` options were specified,
 /// display the requested information and exit.
 #[allow(clippy::print_stdout)]
-fn validate_options(opts: &Cli, config: Config) -> Mode {
-    #[allow(clippy::integer_arithmetic)]
-    (u32::from(opts.query.is_some())
-        + u32::from(opts.match_var_names)
-        + u32::from(opts.list_all)
-        + u32::from(
+fn validate_options(opts: &Cli, config: Config) -> Result<Mode> {
+    ensure!(
+        [
+            opts.query.is_some(),
+            opts.match_var_names,
+            opts.list_all,
             !(config.varnames.is_empty()
                 || opts.match_var_names
                 || opts
                     .query
                     .as_ref()
                     .map_or(false, |value| value == "feature")),
-        )
-        < 2)
-    .or_exit_("Only a single query at a time, please!");
+        ]
+        .into_iter()
+        .filter(|value| *value)
+        .count()
+            < 2,
+        "Only a single query at a time, please!"
+    );
 
     if opts.show_version {
-        let &(_, version) = FEATURES
-            .iter()
-            .find(|&&(name, _)| name == "BASE")
-            .or_exit(|| format!("Internal error: no 'BASE' in {:?}", FEATURES));
-        println!("confget {}", version);
+        let (_, version) = confget::features()
+            .into_iter()
+            .find(|&(name, _)| name == "BASE")
+            .context("Internal error: no 'BASE' in the features list")?;
+        println!("confget {version}");
     }
     if opts.show_help {
-        println!("{}", USAGE);
+        println!("{USAGE}");
     }
     if opts.show_help || opts.show_version {
-        return Mode::Handled(MainResult::OK);
+        return Ok(Mode::Handled(MainResult::OK));
     }
 
     if let Some(ref query) = opts.query {
-        return match query.as_str() {
+        return Ok(match query.as_str() {
             "sections" => {
-                (config.backend == BackendKind::Ini).or_exit_(
+                ensure!(supports_sections_query(&config.backend),
                 "The query for sections is only supported for the 'ini' backend for the present",
             );
                 Mode::Sections(config)
             }
             "features" => {
-                config
-                    .varnames
-                    .is_empty()
-                    .or_exit_("No arguments to -q features");
-                let features: Vec<String> = FEATURES
-                    .iter()
-                    .map(|&(name, version)| format!("{}={}", name, version))
+                ensure!(config.varnames.is_empty(), "No arguments to -q features");
+                let features: Vec<String> = confget::features()
+                    .into_iter()
+                    .map(|(name, version)| format!("{name}={version}"))
                     .collect();
-                println!("{}", features.join(" "));
+                println!("{features}", features = features.join(" "));
                 Mode::Handled(MainResult::OK)
             }
             "feature" => {
                 let (feature_name, rest) = config
                     .varnames
                     .split_first()
-                    .or_exit_("Expected a single feature name");
-                rest.is_empty()
-                    .or_exit_("Only a single feature name expected");
-                match FEATURES.iter().find(|&&(name, _)| name == feature_name) {
-                    Some(&(_, value)) => {
-                        println!("{}", value);
+                    .context("Expected a single feature name")?;
+                ensure!(rest.is_empty(), "Only a single feature name expected");
+                match confget::features()
+                    .into_iter()
+                    .find(|&(name, _)| name == feature_name)
+                {
+                    Some((_, value)) => {
+                        println!("{value}");
                         Mode::Handled(MainResult::OK)
                     }
                     None => Mode::Handled(MainResult::Failure),
                 }
             }
-            other => expect_exit::exit(&format!("Unrecognized query '{}'", other)),
-        };
+            other => bail!(format!("Unrecognized query '{other}'")),
+        });
     }
 
     if opts.list_all {
-        config
-            .varnames
-            .is_empty()
-            .or_exit_("Only a single query at a time, please!");
+        ensure!(
+            config.varnames.is_empty(),
+            "Only a single query at a time, please!"
+        );
     } else if opts.match_var_names {
-        (!config.varnames.is_empty()).or_exit_("No patterns to match against");
+        ensure!(!config.varnames.is_empty(), "No patterns to match against");
     } else {
-        (!config.varnames.is_empty()).or_exit_("No variables specified to query");
+        ensure!(
+            !config.varnames.is_empty(),
+            "No variables specified to query"
+        );
     }
 
     if opts.check_only {
         let (varname, rest) = opts
             .varnames
             .split_first()
-            .or_exit_("Internal error: check_only with no config.varnames");
-        rest.is_empty()
-            .or_exit_("Only a single query at a time, please!");
-        Mode::Check(config, varname.clone())
+            .context("Internal error: check_only with no config.varnames")?;
+        ensure!(rest.is_empty(), "Only a single query at a time, please!");
+        Ok(Mode::Check(config, varname.clone()))
     } else {
-        Mode::Query(config)
+        Ok(Mode::Query(config))
     }
 }
 
 /// Parse the command-line arguments, process informational queries.
-fn parse_args() -> Mode {
+fn parse_args() -> Result<Mode> {
     let opts = Cli::parse();
 
+    let clone_or_empty =
+        |opt: Option<&String>| opt.map_or_else(String::new, |value| (*value).clone());
     let config = Config {
-        backend: opts
-            .backend_name
-            .as_ref()
-            .map_or(BackendKind::Ini, |value| {
-                value.parse().or_exit_e_("Invalid backend specified")
-            }),
+        backend: if let Some(value) = opts.backend_name.as_ref() {
+            value.parse().context("Invalid backend specified")?
+        } else {
+            BackendKind::get_preferred_ini_backend()
+        },
+        encoding: String::new(),
         filename: opts.filename.clone(),
         list_all: opts.list_all,
         match_regex: opts.match_regex,
         match_var_names: opts.match_var_names,
         match_var_values: opts.match_var_values.clone(),
-        name_prefix: match opts.name_prefix {
-            Some(ref value) => value.clone(),
-            None => String::new(),
-        },
-        name_suffix: match opts.name_suffix {
-            Some(ref value) => value.clone(),
-            None => String::new(),
-        },
-        section: match opts.section {
-            Some(ref value) => value.clone(),
-            None => String::new(),
-        },
+        name_prefix: clone_or_empty(opts.name_prefix.as_ref()),
+        name_suffix: clone_or_empty(opts.name_suffix.as_ref()),
+        section: clone_or_empty(opts.section.as_ref()),
         section_override: opts.section_override,
         section_specified: opts.section.is_some(),
         shell_escape: opts.shell_escape,
@@ -302,75 +288,68 @@ fn parse_args() -> Mode {
 
 /// Display the processed variable names and/or values.
 #[allow(clippy::print_stdout)]
-fn output_vars(
-    config: &Config,
-    data: &HashMap<String, HashMap<String, String>>,
-    section: &str,
-) -> MainResult {
+fn output_vars(config: &Config, data: &FileData, section: &str) -> Result<MainResult> {
     for var in format::filter_vars(config, data, section)
-        .or_exit_e_("Could not select the variables to output")
+        .context("Could not select the variables to output")?
     {
-        println!("{}", var.output_full);
+        println!("{output}", output = var.output_full);
     }
-    MainResult::OK
+    Ok(MainResult::OK)
 }
 
 /// Exit with status 0 if the specified variable is present in the config file.
 fn output_check_only(
     _config: &Config,
-    data: &HashMap<String, HashMap<String, String>>,
+    data: &FileData,
     section: &str,
     varname: &str,
 ) -> MainResult {
-    match data.get(section) {
-        Some(sect_data) => {
-            if sect_data.contains_key(varname) {
-                MainResult::OK
-            } else {
-                MainResult::Failure
-            }
+    data.get(section).map_or(MainResult::Failure, |sect_data| {
+        if sect_data.contains_key(varname) {
+            MainResult::OK
+        } else {
+            MainResult::Failure
         }
-        None => MainResult::Failure,
-    }
+    })
 }
 
 /// Output the names of the sections parsed from the config file.
 #[allow(clippy::print_stdout)]
-fn output_sections(data: &HashMap<String, HashMap<String, String>>) -> MainResult {
+fn output_sections(data: &FileData) -> MainResult {
     let mut sections: Vec<&String> = data.keys().filter(|value| !value.is_empty()).collect();
     sections.sort();
     for name in sections {
-        println!("{}", name);
+        println!("{name}");
     }
     MainResult::OK
 }
 
 /// Parse the INI-style file, return the data and the queried section name.
-fn parse_config(config: &Config) -> (HashMap<String, HashMap<String, String>>, String) {
+fn parse_config(config: &Config) -> Result<(FileData, String)> {
     let (data, first_section) =
-        confget::read_ini_file(config).or_exit_e_("Could not initialize the confget parser");
+        confget::read_ini_file(config).context("Could not parse the INI-style file")?;
     let section = if config.section.is_empty() {
         first_section
     } else {
         config.section.clone()
     };
-    (data, section)
+    Ok((data, section))
 }
 
-fn main() -> MainResult {
-    match parse_args() {
+fn main() -> Result<MainResult> {
+    Ok(match parse_args()? {
         Mode::Handled(res) => res,
         Mode::Check(config, varname) => {
-            let (data, section) = parse_config(&config);
+            let (data, section) = parse_config(&config)?;
             output_check_only(&config, &data, &section, &varname)
         }
         Mode::Sections(config) => {
-            let (data, _) = parse_config(&config);
+            let (data, _) = parse_config(&config)?;
             output_sections(&data)
         }
         Mode::Query(config) => {
-            let (data, section) = parse_config(&config);
-            output_vars(&config, &data, &section)
+            let (data, section) = parse_config(&config)?;
+            output_vars(&config, &data, &section)?
         }
-    }
+    })
 }
